"use server";
import { NextResponse } from "next/server";
import { createUser } from "~/server/functions";

type CreateUserRequest = {
  username: string;
  email: string;
  passwordHash: string;
  name: string;
  pfpURL?: string;
  pushToken?: string;
};

export async function POST(request: Request) {
  try {
    const url = new URL(request.url);
    const apiKey = url.searchParams.get("apiKey");

    if (apiKey !== process.env.API_KEY) {
      console.log("Invalid API Key");
      return NextResponse.json({ message: "Invalid API Key" }, { status: 401 });
    }

    // Parse the request body instead of using URL parameters for POST requests
    const body = await request.json() as CreateUserRequest;

    // Destructure the body with default values for optional fields
    const {
      username,
      email,
      passwordHash,
      name,
      pfpURL = "",
      pushToken = ""
    } = body;

    // Validate required fields
    if (!username || !email || !passwordHash || !name) {
      return NextResponse.json(
        { message: "Missing required fields" },
        { status: 400 }
      );
    }

    const user = await createUser(
      username,
      email,
      passwordHash,
      name,
      pfpURL,
      pushToken
    );

    return NextResponse.json(user, { status: 201 });
  } catch (error) {
    console.error("Error creating user:", error);
    if (error instanceof Error) {
      return NextResponse.json({ message: error.message }, { status: 400 });
    }
    return NextResponse.json(
      { message: "An unexpected error occurred" },
      { status: 500 }
    );
  }
}
