import 'server-only';
import { db } from '~/server/db';
import * as schema from '~/server/db/schema';
import { eq, and, or } from 'drizzle-orm';
import { pgEnum } from 'drizzle-orm/pg-core';

// --- Helper Functions --- //

/**
 * Ensure the user exists based on their userId.
 * Throws an error if the user doesn't exist.
 */
export const ensureUserExists = async (userId: number) => {
  try {
    const user = await db.select().from(schema.users)
      .where(eq(schema.users.id, userId));

    if (user.length === 0) {
      throw new Error('User does not exist');
    }

    return user[0]; // Return user for further use if needed
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Error checking user: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while checking user");
    }
  }
};

/** 
 * Ensure relationship exists between a user and a partner.
 * Handles both directions of a relationship (userId => partnerId or partnerId => userId).
 * Optionally checks relationship status.
 */
export const ensureRelationshipExists = async (userId: number, partnerId: number, status?: 'pending' | 'accepted') => {
  try {
    // Ensure bidirectional relationship (user1 <-> user2 or user2 <-> user1)
    const relationship = await db.select({
      relationshipId: schema.userRelationships.relationshipId,
      status: schema.relationships.status,
    })
    .from(schema.userRelationships)
    .leftJoin(
      schema.relationships,
      eq(schema.userRelationships.relationshipId, schema.relationships.id)
    )
    .where(or(
      and(
        eq(schema.userRelationships.userId, userId),
        eq(schema.userRelationships.userId, partnerId) // Check if userId -> partnerId
      ),
      and(
        eq(schema.userRelationships.userId, partnerId),
        eq(schema.userRelationships.userId, userId) // Check if partnerId -> userId (reverse relationship)
      )
    ));

    if (!relationship.length) {
      throw new Error('Relationship does not exist');
    }

    if (status && relationship[0]?.status !== status) {
      throw new Error(`Relationship is not in ${status} status`);
    }

    return relationship[0];
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Error checking relationship: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while checking relationship");
    }
  }
};

// --- User Management Functions --- //

export const getUserById = async (userId: number) => {
  try {
    return await ensureUserExists(userId);
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to fetch user by id: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while fetching user by id");
    }
  }
};

export const getUserByUsername = async (username: string) => {
  try {
    const user = await db.select().from(schema.users)
      .where(eq(schema.users.username, username));

    if (user.length === 0) throw new Error('User not found');
    return user[0];
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to fetch user by username: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while fetching user by username");
    }
  }
};

export const createUser = async (
  username: string, email: string, passwordHash: string,
  name: string, pfpURL = "", pushToken = ""
) => {
  try {
    if (!username || !email || !passwordHash || !name) {
      throw new Error("Error: All required fields must be filled");
    }

    // Check if username or email is already taken
    const existingUser = await db.select().from(schema.users)
      .where(or(eq(schema.users.username, username), eq(schema.users.email, email)));

    if (existingUser.length > 0) {
      throw new Error("Username or email is already in use");
    }

    const newUser = await db.insert(schema.users).values({
      username, email, passwordHash, name, pfpURL, pushToken,
      lastLogin: new Date(),
    }).returning();

    return newUser;
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to create new user: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while creating new user");
    }
  }
};

export const changePassword = async (
  userId: number, oldPasswordHash: string, newPasswordHash: string
) => {
  try {
    // Ensure all arguments are provided
    if (!oldPasswordHash || !newPasswordHash) throw new Error("Password fields are required");

    const user = await ensureUserExists(userId);

    // Validate old password
    if (user?.passwordHash !== oldPasswordHash) {
      throw new Error("Old password does not match");
    }

    // Update with the new password hash
    await db.update(schema.users)
      .set({ passwordHash: newPasswordHash })
      .where(eq(schema.users.id, userId));

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to change password: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while changing password");
    }
  }
};

export const updateUserPFP = async (userId: number, pfpURL: string) => {
  try {
    await db.update(schema.users)
      .set({ pfpURL })
      .where(eq(schema.users.id, userId));

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to update profile: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while updating profile");
    }
  }
};

export const updateUserPushToken = async (userId: number, pushToken: string) => {
  try {
    await db.update(schema.users)
      .set({ pushToken })
      .where(eq(schema.users.id, userId));

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to update profile: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while updating profile");
    }
  }
};

export const userLogin = async (username: string, passwordHash: string) => {
  try {
    const user = await getUserByUsername(username);
    if (user?.passwordHash !== passwordHash) {
      throw new Error("Invalid password");
    }

    // Update last login timestamp
    await db.update(schema.users)
      .set({ lastLogin: new Date() })
      .where(eq(schema.users.id, user.id));

    return user;
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to Log in: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while logging in");
    }
  }
};

// --- Relationship Management Functions --- //

export const createRelationshipRequest = async (requestorId: number, requestedId: number) => {
  try {
    await ensureUserExists(requestorId);
    await ensureUserExists(requestedId);

    // Check if a relationship exists in either direction
    const existingRelationship = await db.select()
      .from(schema.userRelationships)
      .leftJoin(schema.relationships, eq(schema.userRelationships.relationshipId, schema.relationships.id))
      .where(or(
        and(eq(schema.userRelationships.userId, requestorId), eq(schema.userRelationships.userId, requestedId)), // userId -> requestedId
        and(eq(schema.userRelationships.userId, requestedId), eq(schema.userRelationships.userId, requestorId))  // requestedId -> userId
      ));

    if (existingRelationship.length && existingRelationship[0]?.relationship?.status !== 'rejected') {
      throw new Error('A relationship already exists or is pending between these users');
    }

    // Create new relationship entry
    const newRelationship = await db.insert(schema.relationships).values({
      status: "pending",
    }).returning();

    // Check for successful insertion
    if (!newRelationship.length || !newRelationship[0]?.id)
      throw new Error("Failed to create relationship request");

    // Add both users to user_relationships table
    await db.insert(schema.userRelationships).values([
      { userId: requestorId, relationshipId: newRelationship[0]?.id },
      { userId: requestedId, relationshipId: newRelationship[0].id },
    ]);

    return newRelationship;
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to create relationship request: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while creating relationship request");
    }
  }
};


export const updateRelationshipRequest = async (relationshipId: number, status: 'accepted' | 'rejected') => {
  try {
    const relationship = await db.select().from(schema.relationships)
      .where(eq(schema.relationships.id, relationshipId));

    if (!relationship.length) {
      throw new Error("Relationship request not found");
    }

    await db.update(schema.relationships)
      .set({ status })
      .where(eq(schema.relationships.id, relationshipId));

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to update relationship request: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while updating relationship request");
    }
  }
};

export const deleteRelationship = async (relationshipId: number) => {
  try {
    await db.transaction(async (trx) => {
      await trx.delete(schema.userRelationships)
        .where(eq(schema.userRelationships.relationshipId, relationshipId));

      await trx.delete(schema.relationships)
        .where(eq(schema.relationships.id, relationshipId));
    });

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to delete the relationship: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while deleting the relationship");
    }
  }
};

// --- Message Management Functions --- //
type MediaType = "text" | "image" | "video" | "audio" | "file" | "link";
export const sendMessage = async (
  senderId: number, receiverId: number, content: string, mediaUrl?: string, mediaType?: MediaType
) => {
  try {
    // Ensure both sender and receiver exist
    await ensureUserExists(senderId);
    await ensureUserExists(receiverId);

    // Ensure the relationship exists (no need to use returned 'relationshipId' for now)
    await ensureRelationshipExists(senderId, receiverId, "accepted");

    // Insert the new message
    const message = await db.insert(schema.messages).values({
      senderId, receiverId, content, createdAt: new Date(),
    }).returning();

    // Check for successful insertion of the message
    if (!message.length || !message[0]?.id)
      throw new Error("Failed to send message");

    if (mediaUrl && mediaType) {
      await db.insert(schema.messageMedia).values({
        messageId: message[0].id,
        mediaUrl,
        type: mediaType,
      });
    }

    return message;
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to send message: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while sending message");
    }
  }
};

export const fetchMessages = async (userId: number, partnerId: number) => {
  try {
    await ensureUserExists(userId);
    await ensureRelationshipExists(userId, partnerId, "accepted");

    const messages = await db.select().from(schema.messages)
      .where(or(
        and(eq(schema.messages.senderId, userId), eq(schema.messages.receiverId, partnerId)),
        and(eq(schema.messages.senderId, partnerId), eq(schema.messages.receiverId, userId))
      ));

    return messages;
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to retrieve messages: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while retrieving messages");
    }
  }
};

// --- Countdown Management Functions --- //

export const createOrUpdateCountdown = async (relationshipId: number, title: string, date: Date) => {
  try {
    // Check if countdown already exists
    const countdown = await db.select().from(schema.countdowns)
      .where(eq(schema.countdowns.relationshipId, relationshipId));

    // Update existing or insert countdown
    if (countdown.length) {
      await db.update(schema.countdowns)
        .set({ title, date })
        .where(eq(schema.countdowns.relationshipId, relationshipId));
    } else {
      await db.insert(schema.countdowns).values({
        relationshipId, title, date, createdAt: new Date(),
      });
    }

    return { success: true };
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to create/update countdown: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while creating/updating countdown");
    }
  }
};

export const getCountdownByRelationship = async (relationshipId: number) => {
  try {
    const countdown = await db.select().from(schema.countdowns)
      .where(eq(schema.countdowns.relationshipId, relationshipId));

    if (!countdown.length) throw new Error("No countdown found");

    return countdown[0];
  } catch (error) {
    if (error instanceof Error) {
      throw new Error(`Failed to retrieve countdown: ${error.message}`);
    } else {
      throw new Error("Unknown error occurred while retrieving countdown");
    }
  }
};
